//
//  PushPullFlowViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2022/7/28.
//

#import "PushPullFlowViewController.h"
#import <ZegoExpressEngine/ZegoExpressEngine.h>
#import <ZegoAvatar/ZegoAvatarService.h>
#import <ZegoAvatar/ZegoTypes.h>
#import <Masonry/Masonry.h>
#import <Metal/Metal.h>
#import "MetalTools.h"
#import "ZegoAvatarConfig.h"
#import "UIView+Toast.h"



@interface PushPullFlowViewController ()<ZegoEventHandler,ZegoCustomVideoCaptureHandler,ZegoAvatarServiceDelegate, ZegoCustomVideoRenderHandler>{}

@property(nonatomic,strong) UIView *playingPreView;//拉流
@property(nonatomic,strong) UILabel *expresslbl;
@property(nonatomic,strong) UILabel *playinglbl;
@property(nonatomic,strong) NSString *roomID ;
@property(nonatomic,strong) NSString *streamID ;
@property(nonatomic,strong) UIButton *expressbtn;
@property(nonatomic,strong) UIButton *playingbtn;
@property(nonatomic,strong) UIButton *BlueBackgroundButton;
@property(nonatomic,strong) UIButton *RedBackgroundButton;
@property(nonatomic,strong) UIButton *YellowBackgroundButton;
@property(nonatomic,strong) UITextField *publishStreamIDTextField;
@property(nonatomic,strong) UITextField *playStreamIDTextField;
@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character;
@property (nonatomic, strong) id <CAMetalDrawable> currentDrawable;
@property (nonatomic, strong) UIColor* avatarBackgroundColor;
@property (nonatomic, assign) BOOL isStop;

@end
@implementation PushPullFlowViewController

#pragma mark - Setup


- (id)init
{
    if (self = [super init]) {
        // 初始化roomID和streamID,如果想在不同手机上进行推拉流要保证这两个值是相等的，
        // 也就是进入的房间以及确保推拉流是在一条流上
        self.roomID = @"00002";
        self.streamID = @"00001";
        self.isStop = YES;
        _playingPreView = [[UIImageView alloc] init];
        _avatarBackgroundColor = [UIColor whiteColor];
    }
    return self;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}



- (void) initAvatar{
    // 创建 avatarView
    
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake((self.view.bounds.size.width - 300) / 2/*居中*/, getRectNavAndStatusHight + 50, 320, 240)];
    [self.view addSubview:_avatarView];
    
    // 创建一个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    
    [_avatarView setCharacter:_character];
    
    [_character enableAnimation:NO];
    
    [[[ZegoAvatarService sharedInstance] getInteractEngine] startDetectExpression:ZegoExpressionDetectModeCamera callback:^(ZegoExpression *expression) {
        [_character setExpression:expression];
    }];
}

#pragma mark -初始化

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];

    // 创建UI
    [self initView];

    // 初始化express和配置推流设置
    [self initExpressSDK];

}

-(void)initView{
        
    // 推流文字
    _expresslbl = [[UILabel alloc]init];
    _expresslbl.text=@"推流端";
    _expresslbl.textColor=[UIColor blackColor];
    _expresslbl.font=[UIFont systemFontOfSize:15];
    [self.view addSubview:_expresslbl];
    [_expresslbl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.avatarView).mas_offset(20);
        make.width.mas_equalTo(80);
        make.height.mas_equalTo(50);
        make.left.equalTo(self.avatarView).mas_offset(20);
    }];
    

   // 拉流页面
    [self.view addSubview:_playingPreView];
    _playingPreView.backgroundColor = [UIColor systemPinkColor];
    _playingPreView.layer.borderWidth = 1;
    _playingPreView.layer.borderColor = [UIColor blackColor].CGColor;
    [_playingPreView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.avatarView.mas_bottom).mas_offset(50);
        make.width.mas_equalTo(320);
        make.height.mas_equalTo(240);
        make.centerX.equalTo(self.view);
    }];
    // 拉流文字
    _playinglbl = [[UILabel alloc]init];
    _playinglbl.text=@"拉流端";
    _playinglbl.textColor=[UIColor blackColor];
    _playinglbl.font=[UIFont systemFontOfSize:15];
    [self.view addSubview:_playinglbl];
    [_playinglbl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.playingPreView).mas_offset(20);
        make.width.mas_equalTo(80);
        make.height.mas_equalTo(50);
        make.left.equalTo(self.playingPreView).mas_offset(20);
    }];
    

    
    // 拉流端左右镜像问题
    _playingPreView.transform = CGAffineTransformScale(_playingPreView.transform, -1.0, 1.0);//出来以后是左右镜像? 再镜像一下.

    
    UILabel *startlbl = [[UILabel alloc]init];
    startlbl.text=@"推流 StreamID:";
    startlbl.textColor=[UIColor blackColor];
    startlbl.font=[UIFont systemFontOfSize:11];
    startlbl.adjustsFontSizeToFitWidth=YES;
    [self.view addSubview:startlbl];
    [startlbl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.avatarView.mas_top).mas_offset(-10);
        make.width.mas_equalTo(80);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.avatarView);
    }];
//    UITextField *publishStreamIDTextField = [[UITextField alloc]initWithFrame:CGRectMake(20, 100, 130, 30)];
    _publishStreamIDTextField =[[UITextField alloc]init];
    _publishStreamIDTextField.borderStyle=UITextBorderStyleRoundedRect;
    _publishStreamIDTextField.text=@"00001";
    _publishStreamIDTextField.font=[UIFont fontWithName:@"Arial" size:11.0f];
    _publishStreamIDTextField.textColor=[UIColor redColor];
    [self.view endEditing:YES];
    [self.view addSubview:_publishStreamIDTextField];
    [_publishStreamIDTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.avatarView.mas_top).mas_offset(-10);
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(30);
        make.left.equalTo(startlbl.mas_right).mas_offset(20);
    }];
    
    _expressbtn = [self addButtonWithTitle: @"开始推流" action:@selector(startPublishing:)];
    [_expressbtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(70);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.publishStreamIDTextField.mas_right).offset(10);
        make.bottom.equalTo(self.avatarView.mas_top).mas_offset(-10);
    }];

   
    // 拉流端
    UILabel *publishlbl = [[UILabel alloc]init];
    publishlbl.text=@"拉流 StreamID:";
    publishlbl.textColor=[UIColor blackColor];
    publishlbl.font=[UIFont systemFontOfSize:11];
    publishlbl.adjustsFontSizeToFitWidth=YES;
    [self.view addSubview:publishlbl];
    [publishlbl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.avatarView.mas_bottom).mas_offset(10);
        make.width.mas_equalTo(80);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.avatarView);
    }];
    
    _playStreamIDTextField =[[UITextField alloc]init];
    _playStreamIDTextField.borderStyle=UITextBorderStyleRoundedRect;
    _playStreamIDTextField.text=@"00001";
    _playStreamIDTextField.font=[UIFont fontWithName:@"Arial" size:11.0f];
    _playStreamIDTextField.textColor=[UIColor redColor];
    [self.view endEditing:YES];
    [self.view addSubview:_playStreamIDTextField];
    [_playStreamIDTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.avatarView.mas_bottom).mas_offset(10);
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(30);
        make.left.equalTo(publishlbl.mas_right).mas_offset(20);
    }];



    _playingbtn = [self addButtonWithTitle: @"开始拉流" action:@selector(startPlaying:)];
    [_playingbtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(70);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.playStreamIDTextField.mas_right).offset(10);
        make.top.equalTo(self.avatarView.mas_bottom).mas_offset(10);
    }];
    
    _BlueBackgroundButton =[[UIButton alloc]init];
    _BlueBackgroundButton.backgroundColor=[UIColor systemBlueColor];
    _BlueBackgroundButton.layer.cornerRadius=5;
    _BlueBackgroundButton.layer.masksToBounds=YES;
    _BlueBackgroundButton.layer.borderColor=[UIColor blackColor].CGColor;
    _BlueBackgroundButton.layer.borderWidth=1;
    [self.view addSubview:_BlueBackgroundButton];
    [_BlueBackgroundButton addTarget:self action:@selector(beTheBackgroundColor:) forControlEvents:UIControlEventTouchUpInside];
    [_BlueBackgroundButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(30);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.avatarView).mas_offset(10);
        make.top.equalTo(self.expresslbl.mas_bottom).mas_offset(10);
    }];
    
    _RedBackgroundButton =[[UIButton alloc]init];
    _RedBackgroundButton.backgroundColor=[UIColor systemRedColor];
    _RedBackgroundButton.layer.cornerRadius=5;
    _RedBackgroundButton.layer.masksToBounds=YES;
    _RedBackgroundButton.layer.borderColor=[UIColor blackColor].CGColor;
    _RedBackgroundButton.layer.borderWidth=1;
    [self.view addSubview:_RedBackgroundButton];
    [_RedBackgroundButton addTarget:self action:@selector(beTheBackgroundColor:) forControlEvents:UIControlEventTouchUpInside];
    [_RedBackgroundButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(30);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.avatarView).mas_offset(10);
        make.top.equalTo(self.BlueBackgroundButton.mas_bottom).mas_offset(10);
    }];
    
    _YellowBackgroundButton =[[UIButton alloc]init];
    _YellowBackgroundButton.backgroundColor=[UIColor systemYellowColor];
    _YellowBackgroundButton.layer.cornerRadius=5;
    _YellowBackgroundButton.layer.masksToBounds=YES;
    _YellowBackgroundButton.layer.borderColor=[UIColor blackColor].CGColor;
    _YellowBackgroundButton.layer.borderWidth=1;
    
    [self.view addSubview:_YellowBackgroundButton];
    [_YellowBackgroundButton addTarget:self action:@selector(beTheBackgroundColor:) forControlEvents:UIControlEventTouchUpInside];
    [_YellowBackgroundButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(30);
        make.height.mas_equalTo(30);
        make.left.equalTo(self.avatarView).mas_offset(10);
        make.top.equalTo(self.RedBackgroundButton.mas_bottom).mas_offset(10);
    }];
    
    
    
}

// 初始化 express
-(void)initExpressSDK{
    // 创建RTC引擎
    ZegoEngineProfile *profile = [[ZegoEngineProfile alloc] init];
    profile.appID = ZEGO_EXPRESS_APPID;
    profile.appSign = ZEGO_EXPRESS_APPSIGN;
    profile.scenario = ZegoScenarioGeneral;
    [ZegoExpressEngine createEngineWithProfile:profile eventHandler:self];
    
    // 初始话捕获配置
    ZegoCustomVideoCaptureConfig *captureConfig = [[ZegoCustomVideoCaptureConfig alloc] init];
    captureConfig.bufferType = ZegoVideoBufferTypeCVPixelBuffer;

    // 启动自定义视频采集
    [[ZegoExpressEngine sharedEngine] enableCustomVideoCapture:YES config:captureConfig channel:ZegoPublishChannelMain];
    
    // 监听启动和停止自定义采集的事件
    [[ZegoExpressEngine sharedEngine] setCustomVideoCaptureHandler:self];
    // 进入房间, 随便指定一个 userid, 业务里面唯一就行
    ZegoUser *user = [ZegoUser userWithUserID:@"richzhao1"];
    ZegoRoomConfig *config = [ZegoRoomConfig defaultConfig];
    [[ZegoExpressEngine sharedEngine] loginRoom:self.roomID user:user config:config];

    // 设置视频配置, 要跟 avatar 的输出尺寸一致
    ZegoVideoConfig *videoConfig = [ZegoVideoConfig configWithPreset:ZegoVideoConfigPreset720P];
    videoConfig.encodeResolution = CGSizeMake(640, 480);//这是显示区域大小比例.
    [[ZegoExpressEngine sharedEngine] setVideoConfig:videoConfig];
}

// 推流
-(void)startPublishing:(UIButton *)sender{
    if(_isStop){
        [self startPublishing];
        [self.view makeToast:@"开始捕获Avatar并推流"];
    }else{
        [self stopPublishing];
        [self.view makeToast:@"停止捕获Avatar和停止推流"];
    }
}

-(void)startPublishing{
    
    ZegoCanvas *Canvas = [ZegoCanvas canvasWithView:self.avatarView];
    Canvas.viewMode = ZegoViewModeAspectFill;
    
//     express 的预览，因为 avatar 已经上屏，这里不打开了
//    [[ZegoExpressEngine sharedEngine] startPreview:Canvas];
    NSString *publishStreamID = self.publishStreamIDTextField.text;
    // 前置摄像头采集的，设置个拉流端镜像模式
    [[ZegoExpressEngine sharedEngine] setVideoMirrorMode:3];
    // 开始推流
    [[ZegoExpressEngine sharedEngine] startPublishingStream:publishStreamID];
    
    
    __weak __typeof(self) weakSelf = self;
    _isStop = NO;
    // 启动 avatar 捕获
    [self.avatarView setTextureUpdateCallback:^(id<MTLTexture> texture, int width, int height) {
        // 这里获得的texture 纹理类型是 Metal 纹理
        __strong __typeof(self) strongSelf = weakSelf;
        if(strongSelf.isStop){ // 已经停止了就不要推流了
            return;
        }
        @autoreleasepool {
            
            // 捕获到的纹理背景是透明的，可以自行添加背景
            id<MTLTexture> srcTexture = texture;
            [strongSelf getPixelBufferFromBGRAMTLTexture:srcTexture result:^(CVPixelBufferRef pixelBuffer){
                // 创建时间戳
                CMTime time = CMTimeMakeWithSeconds([[NSDate date] timeIntervalSince1970], 1000);
                CMSampleTimingInfo timingInfo = { kCMTimeInvalid, time, time };
                CMVideoFormatDescriptionRef desc;
                CMVideoFormatDescriptionCreateForImageBuffer(kCFAllocatorDefault, pixelBuffer, &desc);

                CMSampleBufferRef sampleBuffer;
                // 包装成 pixelBuffer
                CMSampleBufferCreateReadyWithImageBuffer(kCFAllocatorDefault, (CVImageBufferRef)pixelBuffer, desc, &timingInfo, &sampleBuffer);

                // 使用 express 的 sendCustomVideoCapturePixelBuffer 推流
                [[ZegoExpressEngine sharedEngine] sendCustomVideoCapturePixelBuffer:CMSampleBufferGetImageBuffer(sampleBuffer) timestamp:CMSampleBufferGetPresentationTimeStamp(sampleBuffer)];

                CFRelease(sampleBuffer);
                CFRelease(desc);
            }];
        }// endof @autoreleasepool
    }];
}

-(void)stopPublishing{
    _isStop = YES;
    [[ZegoExpressEngine sharedEngine] stopPreview]; // 停止预览
    [[ZegoExpressEngine sharedEngine] stopPublishingStream]; // 停止推流
    
    // avatar 设置纹理捕获回调接口为 nil
    [self.avatarView setTextureUpdateCallback:nil];
}

-(void)startPlaying:(UIButton *)sender {
    
    
    ZegoCustomVideoRenderConfig *renderConfig = [[ZegoCustomVideoRenderConfig alloc] init];
    // 选择 CVPixelBuffer 类型视频帧数据
    renderConfig.bufferType = ZegoVideoBufferTypeCVPixelBuffer;
    // 选择 RGB 色系数据格式
    renderConfig.frameFormatSeries = ZegoVideoFrameFormatSeriesRGB;
    // 指定在自定义视频渲染的同时引擎也渲染
    renderConfig.enableEngineRender = YES;

    [[ZegoExpressEngine sharedEngine] enableCustomVideoRender:YES config:renderConfig];
    
    [[ZegoExpressEngine sharedEngine] setCustomVideoRenderHandler:self];
    
    ZegoCanvas *CanvasPlaying = [ZegoCanvas canvasWithView:self.playingPreView];
    CanvasPlaying.viewMode = ZegoViewModeAspectFill;
    NSString *playStreamID = self.playStreamIDTextField.text;
    
    [[ZegoExpressEngine sharedEngine] startPlayingStream:playStreamID canvas:CanvasPlaying];
}

// @param channel 推流通道
- (void)onCapturedVideoFrameCVPixelBuffer:(CVPixelBufferRef)buffer param:(ZegoVideoFrameParam *)param flipMode:(ZegoVideoFlipMode)flipMode channel:(ZegoPublishChannel)channel {
    UIImage *img = [PushPullFlowViewController imageFromPixelBuffer:buffer];
    NSLog(@"---");
}

// 远端拉流视频帧 CVPixelBuffer 数据回调，通过 streamID 区分不同的流
//
// @param buffer 封装为 CVPixelBuffer 的视频帧数据
// @param param 视频帧参数
// @param streamID 拉流的流 ID
- (void)onRemoteVideoFrameCVPixelBuffer:(CVPixelBufferRef)buffer param:(ZegoVideoFrameParam *)param streamID:(NSString *)streamID {
    UIImage *img = [PushPullFlowViewController imageFromPixelBuffer:buffer];
    NSLog(@"---");
}

/** pixelBuffer 转 UIImage*/
+ (UIImage *)imageFromPixelBuffer:(CVPixelBufferRef)pixelBuffer {
    CIImage *ciImage = [CIImage imageWithCVPixelBuffer:pixelBuffer];
    
    CIContext *temporaryContext = [CIContext contextWithOptions:nil];
    UIImage *uiImage= nil;
    
    @autoreleasepool {
        CGImageRef videoImage = [temporaryContext
                                 createCGImage:ciImage
                                 fromRect:CGRectMake(0, 0, CVPixelBufferGetWidth(pixelBuffer), CVPixelBufferGetHeight(pixelBuffer))];
        
        uiImage = [UIImage imageWithCGImage:videoImage];
        CGImageRelease(videoImage);
        videoImage = nil;
        
    }
    return uiImage;
    
}

// 触摸view界面收起键盘
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [self.view endEditing:YES]; //实现该方法是需要注意view需要是继承UIControl而来的
}

// 从 metal 纹理常见 pixelBuffer
- (void)getPixelBufferFromBGRAMTLTexture:(id<MTLTexture>)texture result:(void(^)(CVPixelBufferRef pixelBuffer))block {
    
    CVPixelBufferRef pxbuffer = NULL;
    NSDictionary *options = [NSDictionary dictionaryWithObjectsAndKeys:
                             [NSNumber numberWithBool:YES], kCVPixelBufferCGImageCompatibilityKey,
                             [NSNumber numberWithBool:YES], kCVPixelBufferCGBitmapContextCompatibilityKey,
                             nil];
    unsigned long width  = texture.width;
    unsigned long height = texture.height;
    
    
    CVReturn result = CVPixelBufferCreate(kCFAllocatorDefault,
                                          width,
                                          height,
                                          kCVPixelFormatType_32BGRA,
                                          (__bridge CFDictionaryRef) options,
                                          &pxbuffer);
    if (result != kCVReturnSuccess) {
        NSLog(@"Unable to create cvpixelbuffer %d", result);
        return ;
    }
    
    CVPixelBufferLockBaseAddress(pxbuffer, 0);
    void *pxdata = CVPixelBufferGetBaseAddress(pxbuffer);
    size_t bytesPerRow = CVPixelBufferGetBytesPerRow( pxbuffer);
    MTLRegion region = MTLRegionMake2D(0, 0, texture.width, texture.height);
    [texture getBytes:pxdata bytesPerRow:bytesPerRow fromRegion:region mipmapLevel:0];
    CVPixelBufferUnlockBaseAddress(pxbuffer, 0);
    if (block) {
        block(pxbuffer);
    }
    CVPixelBufferRelease(pxbuffer);
}

 
-(void)beTheBackgroundColor:(UIButton *)sender{
 
    self.avatarBackgroundColor = sender.backgroundColor;

 }


#pragma mark - ZegoEventHandler代理

// 房间状态改变通知
- (void)onRoomStateUpdate:(ZegoRoomState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData roomID:(NSString *)roomID {
    if (state == ZegoRoomStateConnected && errorCode == 0) {
        NSLog(@"++==++ 🚩 🚪 Login room success") ;
        
    }
    
    if (errorCode != 0) {
        NSLog(@"++==++ 🚩 ❌ 🚪 Login room fail,error:%d",errorCode) ;
    }
}

// 用户推送音视频流的状态通知
- (void)onPublisherStateUpdate:(ZegoPublisherState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData streamID:(NSString *)streamID {
    if (state == ZegoPublisherStatePublishing && errorCode == 0) {
        NSLog(@"++==++ 🚩 📤 Publishing stream success");
    }
    
    if (errorCode != 0) {
        NSLog(@"++==++ 🚩 ❌ 📤 Publishing stream fail ,error:%d",errorCode);
    }
}

//用户拉取音视频流的状态通知
- (void)onPlayerStateUpdate:(ZegoPlayerState)state errorCode:(int)errorCode extendedData:(NSDictionary *)extendedData streamID:(NSString *)streamID {
    if (state == ZegoPlayerStatePlaying && errorCode == 0) {
        
        NSLog(@"++==++ 🚩 📥 Playing stream success");
        
    }
    
    if (errorCode != 0) {
        
        NSLog(@"++==++ 🚩 ❌ 📥 Playing stream fail,error:%d",errorCode);
        
    }
}


- (void)dealloc {
    // Logout room will automatically stop publishing/playing stream.
//    [[ZegoExpressEngine sharedEngine] logoutRoom:self.roomID];
            
    // Can destroy the engine when you don't need audio and video calls
    //
    // Destroy engine will automatically logout room and stop publishing/playing stream.
    [ZegoExpressEngine destroyEngine:nil];
}



@end


