//
//  VrlImageHelper.m
//  virtualme
//
//  Created by justinsong on 2021/9/24.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "UIImage+Rotate.h"


@implementation VrlImageHelper : NSObject


/** 图片转换方法*/
+(UIImage *_Nullable)imageFromRGBABuffer:(unsigned char*)buffer
                                   width:(int)width
                                  height:(int)heigth
{
    size_t bufferlength = width * heigth * 4;
    CGDataProviderRef provide = CGDataProviderCreateWithData(NULL, buffer, bufferlength, NULL);
    size_t bitsPerComponent = 8;
    size_t bitsPerPixl = 32;
    size_t bytesPerRow = 4 * width;
    CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
    if (colorSpaceRef == NULL) {
        NSLog(@"error allocating color space...");
        CGDataProviderRelease(provide);
        return nil;
    }
    
    CGBitmapInfo bitmapinfo = kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst;
    CGColorRenderingIntent renderingintent = kCGRenderingIntentDefault;
    CGImageRef iref = CGImageCreate(width, heigth, bitsPerComponent, bitsPerPixl, bytesPerRow, colorSpaceRef, bitmapinfo, provide, NULL, YES, renderingintent);
    uint32_t *pixels = (uint32_t*)malloc(bufferlength);
    if (pixels == NULL) {
        NSLog(@"error: memory not allocated for bitmap");
        CGDataProviderRelease(provide);
        CGColorSpaceRelease(colorSpaceRef);
        CGImageRelease(iref);
        return nil;
    }
    
    CGContextRef context = CGBitmapContextCreate(pixels, width, heigth, bitsPerComponent, bytesPerRow, colorSpaceRef, bitmapinfo);
    
    if (context == NULL) {
        NSLog(@"error context not created...");
        free(pixels);
    }
    UIImage *image = nil;
    if (context) {
        CGContextDrawImage(context, CGRectMake(0.0, 0.0, width, heigth), iref);
        CGImageRef imageRef = CGBitmapContextCreateImage(context);
        if ([UIImage respondsToSelector:@selector(imageWithCGImage:scale:orientation:)]) {
            float scale = [[UIScreen mainScreen] scale];
            image = [UIImage imageWithCGImage:imageRef scale:scale orientation:UIImageOrientationUp];
        } else {
            image = [UIImage imageWithCGImage:imageRef];
        }
        
        CGImageRelease(imageRef);
        CGContextRelease(context);
    }
    
    CGColorSpaceRelease(colorSpaceRef);
    CGImageRelease(iref);
    CGDataProviderRelease(provide);
    if(pixels) {
        free(pixels);
    }
    return image;
}


/** 图片转换方法*/
+ (UIImage *_Nullable)imageFromYUVBuffer:(unsigned char*)buffer
                                  width:(int)w
                                 height:(int)h
{
    NSDictionary *pixelAttributes = @{(id)kCVPixelBufferIOSurfacePropertiesKey:@{}};
    CVPixelBufferRef pixelBuffer = NULL;
    CVReturn result = CVPixelBufferCreate(kCFAllocatorDefault,
                                          w,
                                          h,
                                          kCVPixelFormatType_420YpCbCr8BiPlanarFullRange,
                                          (__bridge CFDictionaryRef)(pixelAttributes),
                                          &pixelBuffer);
    
    CVPixelBufferLockBaseAddress(pixelBuffer,0);
    unsigned char *yDestPlane = CVPixelBufferGetBaseAddressOfPlane(pixelBuffer, 0);
    
    // Here y_ch0 is Y-Plane of YUV(NV12) data.
    unsigned char *y_ch0 = buffer;
    
    unsigned char *y_ch1 = buffer + w * h;
    memcpy(yDestPlane, y_ch0, w * h);
    unsigned char *uvDestPlane = CVPixelBufferGetBaseAddressOfPlane(pixelBuffer, 1);
    
    // Here y_ch1 is UV-Plane of YUV(NV12) data.
    memcpy(uvDestPlane, y_ch1, w * h/2);
    CVPixelBufferUnlockBaseAddress(pixelBuffer, 0);
    
    if (result != kCVReturnSuccess) {
        NSLog(@"Unable to create cvpixelbuffer %d", result);
        return nil;
    }
    
    // CIImage Conversion
    CIImage *coreImage = [CIImage imageWithCVPixelBuffer:pixelBuffer];
    CIContext *MytemporaryContext = [CIContext contextWithOptions:nil];
   // size_t size = CVPixelBufferGetDataSize(pixelBuffer);
   // CVPixelBufferGetHeight(pixelBuffer)
   // CVPixelBufferGetWidth(pixelBuffer)
    CGImageRef MyvideoImage = [MytemporaryContext createCGImage:coreImage fromRect:CGRectMake(0, 0, w, h)];
    // UIImage Conversion
    UIImage *Mynnnimage = [[UIImage alloc] initWithCGImage:MyvideoImage scale:1.0 orientation:UIImageOrientationRight];
    
    CVPixelBufferRelease(pixelBuffer);
    CGImageRelease(MyvideoImage);
    return Mynnnimage;
}

/** pixelBuffer 转 UIImage*/
+ (UIImage *)imageFromPixelBuffer:(CVPixelBufferRef)pixelBuffer {
    CIImage *ciImage = [CIImage imageWithCVPixelBuffer:pixelBuffer];
    
    CIContext *temporaryContext = [CIContext contextWithOptions:nil];
    UIImage *uiImage= nil;
    
    @autoreleasepool {
        CGImageRef videoImage = [temporaryContext
                                 createCGImage:ciImage
                                 fromRect:CGRectMake(0, 0, CVPixelBufferGetWidth(pixelBuffer), CVPixelBufferGetHeight(pixelBuffer))];
        
        uiImage = [UIImage imageWithCGImage:videoImage];
        CGImageRelease(videoImage);
        videoImage = nil;
        
    }
    return uiImage;
    
}

/** 读取相机数据，去除对齐像素, 函数内部会为captureBuffer申请内存，外部负责释放！*/
+ (bool)bufferFromPixelBuffer:(CVPixelBufferRef)pixelBuffer imageBuffer:(unsigned char **)captureBuffer width:(int *)width height:(int *)height channel:(int)channel {
    
    
    if (pixelBuffer == NULL) {
        NSLog(@"CameraView: readBufferWithoutEdgePixel, _pixelBuffer havent return yet");
        return false;
    }
    
    if (captureBuffer == NULL) {
        NSLog(@"CameraView: readBufferWithoutEdgePixel, havent init camera yet");
        return false;
    }
    
    int bufferWidth = (int)CVPixelBufferGetWidth( pixelBuffer );
    int bufferHeight = (int)CVPixelBufferGetHeight( pixelBuffer );
    *width = bufferWidth;
    *height = bufferHeight;
    int srcLen = (int)CVPixelBufferGetDataSize(pixelBuffer);
    *captureBuffer = (unsigned char *)malloc(srcLen);
    CVPixelBufferLockBaseAddress(pixelBuffer, 0);
    
    /**源*/
    size_t srcRowBytes = CVPixelBufferGetBytesPerRow( pixelBuffer );
    uint8_t *srcBuffer = (uint8_t *)CVPixelBufferGetBaseAddress( pixelBuffer );

    /**目标*/
    long destPixelBytes = channel;
    size_t destRowBytes = bufferWidth * destPixelBytes;
    unsigned char *destBuffer = *captureBuffer;

    /**逐行拷贝, 跳过无用的对齐像素，否则图像会扭曲*/
    for ( int row = 0; row < bufferHeight; row++ )
    {
        uint8_t *srcAddress = srcBuffer + (row * srcRowBytes);
        uint8_t *destAddress = destBuffer + (row * destRowBytes);
        memcpy(destAddress, srcAddress, destRowBytes);
    }

    CVPixelBufferUnlockBaseAddress(pixelBuffer, 0);
    
    return true;
}


/** pixelbuffer(YUV) -> buffer(BGRA) 函数内部会为captureBuffer申请内存，外部负责释放！*/
+ (bool)bgraBufferFromYUVPixelBuffer:(CVPixelBufferRef)pixelBuffer imageBuffer:(unsigned char **)captureBuffer width:(int *)width height:(int *)height channel:(int)channel {
    NSString *imagePath;
    UIImage *image = [self imageFromPixelBuffer:pixelBuffer];
    UIImage *poriateImage = [image rotate:UIImageOrientationRight];
    CVPixelBufferRef pixelBufferBGRA = [self imageToBGRAPixelBuffer:poriateImage];
    return [self bufferFromPixelBuffer:pixelBufferBGRA imageBuffer:captureBuffer width:width height:height channel:4];
}

/** UIimage-> buffer(BGRA) 函数内部会为captureBuffer申请内存，外部负责释放！*/
+ (bool)bgraBufferFromImage:(UIImage *)image imageBuffer:(unsigned char **)captureBuffer width:(int *)width height:(int *)height channel:(int)channel {
    CVPixelBufferRef pixelBufferBGRA = [self imageToBGRAPixelBuffer:image];
    return [self bufferFromPixelBuffer:pixelBufferBGRA imageBuffer:captureBuffer width:width height:height channel:channel];
}



+ (UIImage *)imageFromYUV:(int)w h:(int)h buffer:(unsigned char *_Nullable)buffer
{
    NSDictionary *pixelAttributes = @{(id)kCVPixelBufferIOSurfacePropertiesKey:@{}};
    CVPixelBufferRef pixelBuffer = NULL;
    CVReturn result = CVPixelBufferCreate(kCFAllocatorDefault,
                                          w,
                                          h,
                                          kCVPixelFormatType_420YpCbCr8BiPlanarFullRange,
                                          (__bridge CFDictionaryRef)(pixelAttributes),
                                          &pixelBuffer);
    
    CVPixelBufferLockBaseAddress(pixelBuffer,0);
    unsigned char *yDestPlane = CVPixelBufferGetBaseAddressOfPlane(pixelBuffer, 0);
    
    // Here y_ch0 is Y-Plane of YUV(NV12) data.
    unsigned char *y_ch0 = buffer;
    
    unsigned char *y_ch1 = buffer + w * h;
    memcpy(yDestPlane, y_ch0, w * h);
    unsigned char *uvDestPlane = CVPixelBufferGetBaseAddressOfPlane(pixelBuffer, 1);
    
    // Here y_ch1 is UV-Plane of YUV(NV12) data.
    memcpy(uvDestPlane, y_ch1, w * h/2);
    CVPixelBufferUnlockBaseAddress(pixelBuffer, 0);
    
    if (result != kCVReturnSuccess) {
        NSLog(@"Unable to create cvpixelbuffer %d", result);
        return nil;
    }
    
    // CIImage Conversion
    CIImage *coreImage = [CIImage imageWithCVPixelBuffer:pixelBuffer];
    CIContext *MytemporaryContext = [CIContext contextWithOptions:nil];
   // size_t size = CVPixelBufferGetDataSize(pixelBuffer);
   // CVPixelBufferGetHeight(pixelBuffer)
   // CVPixelBufferGetWidth(pixelBuffer)
    CGImageRef MyvideoImage = [MytemporaryContext createCGImage:coreImage fromRect:CGRectMake(0, 0, w, h)];
    // UIImage Conversion
    UIImage *Mynnnimage = [[UIImage alloc] initWithCGImage:MyvideoImage scale:1.0 orientation:UIImageOrientationRight];
    
    CVPixelBufferRelease(pixelBuffer);
    CGImageRelease(MyvideoImage);
    return Mynnnimage;
}

/** */
+ (CVPixelBufferRef)imageToBGRAPixelBuffer:(UIImage *)image {
    CGSize frameSize = CGSizeMake(CGImageGetWidth(image.CGImage),CGImageGetHeight(image.CGImage));
    NSDictionary *options =
    [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],kCVPixelBufferCGImageCompatibilityKey,[NSNumber numberWithBool:YES],kCVPixelBufferCGBitmapContextCompatibilityKey,nil];
    CVPixelBufferRef pxbuffer = NULL;
    CVReturn status =
    CVPixelBufferCreate(kCFAllocatorDefault, frameSize.width, frameSize.height,kCVPixelFormatType_32BGRA, (__bridge CFDictionaryRef)options, &pxbuffer);
    NSParameterAssert(status == kCVReturnSuccess && pxbuffer != NULL);
    CVPixelBufferLockBaseAddress(pxbuffer, 0);
    void *pxdata = CVPixelBufferGetBaseAddress(pxbuffer);
    CGColorSpaceRef rgbColorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(pxdata, frameSize.width, frameSize.height,8, CVPixelBufferGetBytesPerRow(pxbuffer),rgbColorSpace,(CGBitmapInfo)kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    CGContextDrawImage(context, CGRectMake(0, 0, CGImageGetWidth(image.CGImage),CGImageGetHeight(image.CGImage)), image.CGImage);
    CGColorSpaceRelease(rgbColorSpace);
    CGContextRelease(context);
    CVPixelBufferUnlockBaseAddress(pxbuffer, 0);
    return pxbuffer;
}

@end
