//
//  ZegoExpressAudioCaptureDelegate.m
//  GoAvatar
//
//  Created by Azrael Long on 2022/2/8.
//

#import "ZegoExpressAudioCaptureDelegate.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <ZegoAvatar/IZegoInteractEngine.h>
#import <ZegoAvatar/AudioDataDelegate.h>
#import "ZegoExpressAudioCaptureDelegate.h"
#import <ZegoExpressEngine/ZegoExpressEngine.h>
#import "ZegoAvatarConfig.h"

@interface ZegoExpressAudioCaptureDelegate()<ZegoEventHandler, ZegoCustomAudioProcessHandler>
{
    bool _isRunning;
    bool _isInit;
}

@end

@implementation ZegoExpressAudioCaptureDelegate

- (id)init
{
    self = [super init];
    if (self) {
        _isRunning = false;
    }
    return self;
}

- (void)dealloc
{
    [self destroyExpress];
}



#pragma mark - ZegoExpress
- (void) initExpress{
    if(_isInit){
        return;
    }
    _isInit = true;
    ZegoEngineProfile *profile = [ZegoEngineProfile new];
    // 请通过官网注册获取，格式为：1234567890
    profile.appID = ZEGO_EXPRESS_APPID;
    //请通过官网注册获取，格式为：@"0123456789012345678901234567890123456789012345678901234567890123"（共64个字符）
    profile.appSign = ZEGO_EXPRESS_APPSIGN;
    //通用场景接入
    profile.scenario = ZegoScenarioGeneral;
    // 创建引擎，并注册 self 为 eventHandler 回调。不需要注册回调的话，eventHandler 参数可以传 nil，后续可调用 "-setEventHandler:" 方法设置回调
    [ZegoExpressEngine createEngineWithProfile:profile eventHandler:self];
    
    
    // 开启 AEC
    [[ZegoExpressEngine sharedEngine] enableAEC:YES];
    // 在使用耳机时开启 AEC
    [[ZegoExpressEngine sharedEngine] enableHeadphoneAEC:YES];
    // 设置 AEC 模式为 ZegoAECModeMedium
    [[ZegoExpressEngine sharedEngine] setAECMode:ZegoAECModeMedium];
    
    // 开启 AGC
    [[ZegoExpressEngine sharedEngine] enableAGC:YES];
    
    // 开启 ANS
    [[ZegoExpressEngine sharedEngine] enableANS:YES];
    // 开启瞬态噪声抑制
    [[ZegoExpressEngine sharedEngine] enableTransientANS:YES];
    // 设置 ANS 模式为 ZegoANSModeMedium
    [[ZegoExpressEngine sharedEngine] setANSMode:ZegoANSModeMedium];
    
    
    // 注册自定义音频前处理
    [[ZegoExpressEngine sharedEngine] setCustomAudioProcessHandler:self];
     
    // 启用rtc的自定义前处理
    ZegoCustomAudioProcessConfig *config = [[ZegoCustomAudioProcessConfig alloc] init];
    config.channel = ZegoAudioChannelMono;
    config.sampleRate = ZegoAudioSampleRate16K;
    config.samples = 0;
    [[ZegoExpressEngine sharedEngine] enableCustomAudioCaptureProcessing:YES config:config];
}

// 开始推流, 接收 rtc 的音频数据和声浪
- (void) startPush{
    // 开启预览
    [[ZegoExpressEngine sharedEngine] startPreview: nil];
    printf("[speech]: use rtc record\n");
    
}

- (void) stopPush{
    printf("[speech]: stop record\n");
    [[ZegoExpressEngine sharedEngine] stopPreview];
    
    
}

- (void)destroyExpress{
    [[ZegoExpressEngine sharedEngine] setCustomAudioProcessHandler:nil];
    [ZegoExpressEngine destroyEngine: nil];
}


- (void)onStart{
    // Avatar SDK 开始录制/音频驱动, 如果RTC没启动, 那就要启动采样了
    [self initExpress];
    if(!_isRunning){
        [self startPush]; // 进房和推流
    }
    _isRunning = true;
}

- (void)onStop{
    // Avatar SDK 已经不需要音频了, RTC逻辑里自行判断是否要关闭
    if(_isRunning){
        [self stopPush]; // 停止推流
        _isRunning = false;
    }
}

// 自定义声音的前处理回调, 把数据发送给 Avatar SDK
- (void)onProcessCapturedAudioData:(unsigned char * _Nonnull)data dataLength:(unsigned int)dataLength param:(ZegoAudioFrameParam *)param timestamp:(double)timestamp; {
    
    @autoreleasepool {
        if(_isRunning){
            // sendAudioData 是父类方法, 数据透传给 Avatar SDK, dataType 的取值有 0 和 1::0: 代表 16位的int; 1: 8位的int;
            [super sendAudioData: (void*)data  size:dataLength dataType: 1 /* RTC 给的 8bit*/ timeStamp: [super getDurationMs]];
        }
        else
        {
            NSLog(@"onProcessCapturedAudioData rtc has STOP !");
        }
    }
    

}


@end
