//
//  GaussianBlurFilter.h
//  ZegoAvatarExample
//
//  Created by applechang on 2022/9/28.
//

#import "GaussianBlurFilter.h"
#import <Foundation/Foundation.h>
#import <Metal/Metal.h>
#import <MetalPerformanceShaders/MetalPerformanceShaders.h>

@interface GaussianBlurFilter () {
    id<MTLDevice> _device;
    id<MTLRenderPipelineState> _pipelineState;
    MTLRenderPassDescriptor *_renderPassDes;
    id<MTLCommandQueue> _commandQueue;
    MTLTextureDescriptor *_texDescriptor;
    UIImage *_lastBgImage;
    id<MTLTexture> _bgTexture;
}

@property (nonatomic, strong) MPSUnaryImageKernel *gaussianBlurKernel;
@end

/// 这个类用来做图片和avatar合成的功能, metal的主要流程放这里
@implementation GaussianBlurFilter

-(instancetype)init{
    if(self = [super init]){
        [self setupPineline];
    }
    
    return self;
}

- (void)setupPineline {
    _device = MTLCreateSystemDefaultDevice();
    _commandQueue = [_device newCommandQueue];
    
    self.gaussianBlurKernel = [[MPSImageGaussianBlur alloc] initWithDevice: _device sigma: 30];
    [self.gaussianBlurKernel setEdgeMode: MPSImageEdgeModeClamp];
}

- (void)startGaussianBlur:(id<MTLTexture>)inTexture complete:(OnGaussianBlurCallback)callback{
    id<MTLCommandBuffer> commandBuffer = [_commandQueue commandBuffer];
    commandBuffer.label = @"GaussianBlurMPSCommand";
    
    MTLTextureDescriptor *textureDescriptor = [[MTLTextureDescriptor alloc] init];
    textureDescriptor.textureType = inTexture.textureType;
    textureDescriptor.pixelFormat = inTexture.pixelFormat;
    textureDescriptor.width = inTexture.width;
    textureDescriptor.height = inTexture.height;
    textureDescriptor.usage = inTexture.usage;
    textureDescriptor.usage = MTLTextureUsageShaderRead | MTLTextureUsageShaderWrite;

    id<MTLTexture> outTexture =  [_device newTextureWithDescriptor:textureDescriptor];
    
    [self encodeToCommandBuffer:commandBuffer srcTexture:inTexture dstTexture:outTexture];
    
    [commandBuffer addCompletedHandler:^(id<MTLCommandBuffer> _Nonnull commandBuffer) {
      if (callback != nil) {
          callback(outTexture);
      }
    }];
    
    [commandBuffer commit];
}

- (void)encodeToCommandBuffer: (id<MTLCommandBuffer>) commandBuffer
                   srcTexture: (id<MTLTexture>) srcTexture
                   dstTexture: (id<MTLTexture>) dstTexture{
    if(!commandBuffer){
        NSLog(@"invalid commandBuffer");
        return ;
    }
    if(!srcTexture || !dstTexture){
        NSLog(@"invalid texture");
        return ;
    }
    if(srcTexture.width != dstTexture.width || srcTexture.height != dstTexture.height){
        NSLog(@"invalid size for blending");
        return ;
    }
    
    [_gaussianBlurKernel encodeToCommandBuffer: commandBuffer sourceTexture: srcTexture destinationTexture: dstTexture];
}
@end
