//
//  ZegoTextAPI.h
//  GoAvatar
//
//  Created by lihua.wei on 2023/2/7.
//

#import <Foundation/Foundation.h>
#import "ZegoTextRequest.h"

typedef NS_ENUM(NSUInteger, ZegoTextAPIErrorCode) {
    // 成功
    ZegoTextAPIErrorCode_Success = 0,
    
    // 错误的参数
    ZegoTextAPIErrorCode_ErrInvalidParameters = 4,
    
    // appID 不可用
    ZegoTextAPIErrorCode_ErrAppIdNotAvailable = 5,
    
    // 非法签名
    ZegoTextAPIErrorCode_ErrInvalidSign = 6,
    
    // appID 校验失败，请检查签名
    ZegoTextAPIErrorCode_ErrInvalidTokenOrSign = 8,
    
    // 签名过期，请重新生成
    ZegoTextAPIErrorCode_ErrInvalidSignExpire = 9,
    
    // 不是毫秒级时间戳
    ZegoTextAPIErrorCode_ErrInvalidTimestamp = 10,
    
    // token过期，请重新生成
    ZegoTextAPIErrorCode_ErrInvalidTokenExpire = 11,
    
    // 文本过长
    ZegoTextAPIErrorCode_ErrInvalidTextLen = 10001,
    
    // 传入文本非utf8编码
    ZegoTextAPIErrorCode_ErrInvalidTextNotUTF8 = 10002,
    
    // voice_type非法，请参考音色列表
    ZegoTextAPIErrorCode_ErrInvalidVoiceType = 10003,
    
    // 参数FPS错误
    ZegoTextAPIErrorCode_ErrInvalidFps = 10004,
    
    // 参数volume音量错误
    ZegoTextAPIErrorCode_ErrInvalidVolume = 10005,
    
    // 参数speed语速错误
    ZegoTextAPIErrorCode_ErrInvalidSpeed = 10006,
    
    // 请求文本含有非法字符，或请求文本没有有效字符
    ZegoTextAPIErrorCode_ErrInvalidIllegalText = 20001,
    
    // 初始化没有传入 Avatar 的 character。
    ZegoTextAPIErrorCode_ErrInvalidCharacterNil = 90004,
    
    // playTextExpression 的 参数 text 为空。
    ZegoTextAPIErrorCode_ErrInvalidTextNil = 90005,
    
    // playTextExpression 的 参数 text 过长。
    ZegoTextAPIErrorCode_ErrInvalidTextOverlong = 90006,
    
};



@protocol ITextExpressionDelegate <NSObject>

@optional

// 文本驱动开始时回调。
- (void)onStart;

// 文本驱动发生错误时回调。
- (void)onError:(ZegoTextAPIErrorCode)errorCode message:(NSString *_Nullable)message;

// 文本驱动结束时回调。
- (void)onEnd;

@end

@protocol IZegoCharacter;

NS_ASSUME_NONNULL_BEGIN

@interface ZegoTextAPI : NSObject

// 代理
@property (nonatomic, weak) id <ITextExpressionDelegate> delegate;

// 初始化
- (instancetype)initWithCharacter:(id<IZegoCharacter>)character;

// 开始播放
- (void)playTextExpression:(NSString *)text;

// 停止播放
- (void)stopTextExpression;

@end

NS_ASSUME_NONNULL_END
