//
//  ZegoTextRequest.m
//  GoAvatar
//
//  Created by lihua.wei on 2023/2/7.
//

#import "ZegoTextRequest.h"
#import <AFNetworking/AFHTTPSessionManager.h>
#import <YYKit/NSObject+YYModel.h>

@implementation ZegoTextDataModel

@end

@implementation ZegoTextModel

+ (nullable NSDictionary<NSString *, id> *)modelContainerPropertyGenericClass {
    return @{
        @"data" : [ZegoTextDataModel class],
    };
}

@end


@interface ZegoTextRequest ()

@property (nonatomic, assign) NSUInteger appID;
@property (nonatomic, strong) NSString *appSign;
@property (nonatomic, strong) NSString *text;
@property (nonatomic, assign) NSUInteger timestamp;
@property (nonatomic, assign) NSUInteger voiceType;
@property (nonatomic, assign) NSUInteger language;
@property (nonatomic, assign) NSUInteger sampleRate;
@property (nonatomic, strong) NSString *codec;
@property (nonatomic, strong) NSString *nonce;
@property (nonatomic, assign) NSUInteger speed;

@end

@implementation ZegoTextRequest

- (void)requestBsWithText:(NSString *)text
                    appID:(NSUInteger)appID
                  appSign:(NSString *)appSign
                timestamp:(NSUInteger)timestamp
                voiceType:(NSUInteger)voiceType
                 language:(NSUInteger)language
               sampleRate:(NSUInteger)sampleRate
                    codec:(NSString *)codec
                    nonce:(NSString *)nonce
                    speed:(NSUInteger)speed
                completed:(void(^)(ZegoTextModel *model))completed {
    
    self.text = text;
    self.appID = appID;
    self.appSign = appSign;
    self.timestamp = timestamp;
    self.voiceType = voiceType;
    self.language = language;
    self.sampleRate = sampleRate;
    self.codec = codec;
    self.nonce = nonce;
    self.speed = speed;
    
    [self startRequestWithUrl:self.baseUrl method:@"POST" params:self.requestArgument completed:completed];
}

- (NSString *)baseUrl {
    return @"https://general-ai-gugong-smprod-base.zego.im/general/text_driven";
}

- (id)requestArgument {
    return @{
        @"text" : self.text,
        @"app_id" : @(self.appID),
        @"sign" : self.appSign,
        @"timestamp" : @(self.timestamp),
        @"primary_language" : @(self.language),
        @"codec" : self.codec,
        @"sample_rate" : @(self.sampleRate),
        @"voice_type" : @(self.voiceType),
        @"nonce" : self.nonce,
        @"speed" : @(self.speed),
    };
}

- (void)startRequestWithUrl:(NSString *)url method:(NSString *) method params:(NSDictionary *)params completed:(void(^)(ZegoTextModel *model))completed {
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:params options:0 error:&error];
    NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    // 使用到AFNetworking
    AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    manager.responseSerializer = [AFJSONResponseSerializer serializer];
    //method 为时post请求还是get请求
    NSMutableURLRequest *request = [[AFJSONRequestSerializer serializer] requestWithMethod:method URLString:url parameters:nil error:nil];
    //设置超时时长
    request.timeoutInterval= 30;
    //设置上传数据type
    [request setValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
    //设置接受数据type
    [request setValue:@"application/json" forHTTPHeaderField:@"Accept"];

    //将对象设置到requestbody中 ,主要是这不操作
    [request setHTTPBody:[jsonString dataUsingEncoding:NSUTF8StringEncoding]];
    //进行网络请求
    [[manager dataTaskWithRequest:request uploadProgress:nil downloadProgress:nil completionHandler:^(NSURLResponse * _Nonnull response, id  _Nullable responseObject, NSError * _Nullable error) {
        if (!error) {
            // 使用到YYKit
            ZegoTextModel *model = [ZegoTextModel modelWithJSON:responseObject];
            if (completed) {
                completed(model);
            }
        } else {
            NSLog(@"Error: %@, %@, %@", error, response, responseObject);
            if (completed) {
                completed(nil);
            }
        }
    }] resume];
}

@end
