//
//  MultiAnimationViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2023/5/11.
//

#import "AnimationViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>

@interface AnimationViewController()

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character1;
@property (nonatomic, strong) id<IZegoCharacter> character2;
@property (nonatomic, strong) UIButton *playAnimBtn;
@property (nonatomic, strong) UIButton *playMultiAnimBtn;
@end

@implementation AnimationViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];
    
    //切换按钮
    _playAnimBtn = [self addButtonWithTitle:@"单人动画" action:@selector(playAnim)];
    [_playAnimBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view.mas_centerX).offset(-10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
    _playAnimBtn.enabled = YES;

    //切换按钮
    _playMultiAnimBtn = [self addButtonWithTitle:@"多人动画" action:@selector(playMultiAnim)];
    [_playMultiAnimBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view.mas_centerX).offset(10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
    _playMultiAnimBtn.enabled = NO;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}


-(void)playAnim {
    [_avatarView removeCharacter:_character2];
    [_character1 playAnimation:[self getPackagePath:@"ZEGOQ_Man_Action_Blowkiss_0001"]];
    _playAnimBtn.enabled = NO;
    _playMultiAnimBtn.enabled = YES;
}

- (void)playMultiAnim {
    NSString *assetBundlesGirlPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Girl.prefab"];
    _character2 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesGirlPath];
    [_avatarView setCharacter:_character2];
    NSString *action1 = @"ZEGOQ_Action_Both_Hug_A_0001";
    NSString *action2 = @"ZEGOQ_Action_Both_Hug_B_0001";
    
    NSMutableDictionary<NSString *, NSString *> *map = [[NSMutableDictionary alloc] init];
    [map setValue:[self getPackagePath:action1] forKey:[self.character1 getRoleID]];
    [map setValue:[self getPackagePath:action2] forKey:[self.character2 getRoleID]];
    __weak typeof(self) weakSelf = self;
    [self.avatarView playMultiCharacterAnimation:map callback:^(int error) {
        __strong typeof(self) he = weakSelf;
        [he.avatarView removeCharacter:self.character2];
    }];
    _playAnimBtn.enabled = YES;
    _playMultiAnimBtn.enabled = NO;
}


#pragma mark - Avatar

// 初始化 avatar
- (void) initAvatar{
    NSLog(@"AnimationViewController init avatar!!");
    
    // 初始化 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 630)];
    [self.view addSubview:_avatarView];
    _avatarView.backgroundColor = [UIColor brownColor];
    
    // 创建两个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character1 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    [_avatarView setCharacter:_character1];
}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/Packages/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}
@end
