//
//  CameraMotionViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2023/11/15.
//

#import "CameraMotionViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>


@interface CameraMotionViewController()
@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character1;
@property (nonatomic, strong) UIButton *playCameraMotion;
@property (nonatomic, strong) UIButton *stopCameraMotion;
@property (nonatomic, strong) NSArray *stringArray;
@end

@implementation CameraMotionViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    
    _stringArray = @[@"cameraMotion_l2f", @"cameraMotion_r2f", @"cameraMotion_f2n", @"cameraMotion_n2f",  @"pkkr_start_1"];
    
    // 初始化 Avatar
    [self initAvatar];
    
    //切换按钮
    _playCameraMotion = [self addButtonWithTitle:@"播放运镜" action:@selector(playCameraMotionClick)];
    [_playCameraMotion mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view.mas_centerX).offset(-10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
    
    _stopCameraMotion = [self addButtonWithTitle:@"停止运镜" action:@selector(stopCameraMotionClick)];
    [_stopCameraMotion mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view.mas_centerX).offset(10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}


-(void)playCameraMotionClick {
    // 生成一个随机索引
    NSUInteger randomIndex = arc4random_uniform((uint32_t)[self.stringArray count]);
    // 使用索引获取随机字符串
    NSString *randomString = self.stringArray[randomIndex];
    // 播放运镜
    NSString *motionPath = [[NSBundle mainBundle] pathForResource:randomString ofType:@"json"];
    NSData *data = [[NSData alloc] initWithContentsOfFile:motionPath];
    NSString *motionStr = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    [_avatarView  playCameraMotion:motionStr callback:^(int errorCode) {
        // 播放结束回调
    }];
}

-(void)stopCameraMotionClick {
    [_avatarView stopCameraMotion];
}



#pragma mark - Avatar

// 初始化 avatar
- (void) initAvatar{
    NSLog(@"AnimationViewController init avatar!!");
    
    // 初始化 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 630)];
    [self.view addSubview:_avatarView];
    _avatarView.backgroundColor = [UIColor brownColor];
    
    // 创建两个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character1 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    [_avatarView setCharacter:_character1 callback:^{
       // 上屏回调
    }];
}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/Packages/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}
@end

