//
//  DressupViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2022/7/7.

#import "DressupViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>
#import "UIView+Toast.h"

@interface DressupViewController ()<ZegoAvatarServiceDelegate> {
}

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character;
@property (nonatomic, strong)UIButton *earphone;
@property (nonatomic, strong)UIButton *glasses;
@property (nonatomic, strong)UIButton *hair;
@property (nonatomic, strong)UIButton *facepaint;
@property (nonatomic, strong) UIButton *faceDetectBtn;
@property (nonatomic, strong) UIButton *voiceDetectBtn;
@property (nonatomic, assign) BOOL isEarphoneEnable;
@property (nonatomic, assign) BOOL isHairEnable;
@property (nonatomic, assign) BOOL isGlassEnable;
@property (nonatomic, assign) BOOL isFacepaintEnable;
@property (nonatomic, strong) NSString *makeupConfig;
@end

@implementation DressupViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];
    
    NSInteger count = 3;
    CGFloat margin = 15;
    CGFloat btnWidth = (self.view.bounds.size.width - 4 * margin) / 3;
    CGFloat btnHeight = 44;
    NSArray *sourceArr = @[@"耳机", @"眼镜", @"头发", @"刺青", @"脸型", @"眼型", @"上衣", @"裤子", @"肤色", @"唇色", @"发色", @"获取妆容数据"];
    for (int i = 0; i < sourceArr.count; i++) {
        CGFloat top = getRectNavAndStatusHight + self.view.bounds.size.width + margin + (margin + btnHeight) * (i / count);
        CGFloat left = margin + (margin + btnWidth) * (i % count);
        NSString *source = sourceArr[i];
        UIButton *btn = [self addButtonWithTitle:source action:@selector(buttonClick:)];
        btn.tag = i;
        [btn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.width.mas_equalTo(btnWidth);
            make.height.mas_equalTo(btnHeight);
            make.left.mas_equalTo(left);
            make.top.mas_equalTo(top);
        }];
    }
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}

- (void)buttonClick:(UIButton *)sender {
    sender.selected = !sender.selected;
    switch (sender.tag) {
        case 0: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Headset_0001_0001"]];
            } else{
                [_character removePackage:ZegoAvatarMakeupTypeHeadset];
            }
        }
            break;
            
        case 1: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Glass_0001_0001"]];
            } else{
                [_character removePackage:ZegoAvatarMakeupTypeGlasses];
            }
        }
            break;
        case 2: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Hair_0002_0001"]];
            } else{
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Hair_0001_0001"]];
            }
        }
            break;
        case 3: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_W_Sticker_0005_0001_Basecolor"]];
            } else{
                [_character removePackage:ZegoAvatarMakeupTypeFacePaint];
            }
        }
            break;
        case 4: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Faceshape_Pointed_Man"]];
            } else{
                [_character removePackage:ZegoAvatarMakeupTypeFaceShape];
            }
        }
            break;
        case 5: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_EyeShape_XiaChui_Man"]];
            } else{
                [_character removePackage:ZegoAvatarMakeupTypeEyeShape];
            }
        }
            break;
        case 6: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Coat_0003_0002"]];
            } else{
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Coat_0004_0004"]];
            }
        }
            break;
        case 7: {
            if(sender.selected) {
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Pants_0003_0005"]];
            } else{
                [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Pants_0006_0004"]];
            }
        }
            break;
        case 8: {
            int r = arc4random() % 255;
            int g = arc4random() % 255;
            int b = arc4random() % 255;
            NSString *skinColor = [NSString stringWithFormat:@"%d,%d,%d,255", r, g, b];
            [self.character setSkinColor:skinColor];
        }
            break;
        case 9: {
            int r = arc4random() % 255;
            int g = arc4random() % 255;
            int b = arc4random() % 255;
            NSString *lipColor = [NSString stringWithFormat:@"%d,%d,%d,255", r, g, b];
            [self.character setLipColor:lipColor];
        }
            break;
        case 10: {
            int r = arc4random() % 255;
            int g = arc4random() % 255;
            int b = arc4random() % 255;
            NSString *hairColor = [NSString stringWithFormat:@"%d,%d,%d,255", r, g, b];
            // 这是设置发色的接口，请注意，有头发设置发色才有效哦
            [self.character setHairColor:hairColor endColor:hairColor];
        }
            break;
        case 11: {
            self.makeupConfig = [self.character getMakeupConfig];
            [self.view makeToast:[NSString stringWithFormat:@"妆容数据：%@", self.makeupConfig]];
        }
            break;
        default:
            break;
    }
}


#pragma mark - Avatar

- (void) initAvatar{
   // 创建 avatarView
   _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, getRectNavAndStatusHight, self.view.bounds.size.width, self.view.bounds.size.width)];
   [self.view addSubview:_avatarView];
    
    // 创建一个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    
    // 把角色渲染到 view 上，不然不可见
    [_avatarView setCharacter:_character];
}

//耳机点击事件
-(void)Addearphone:(UIButton *)sender {
// 如果点击耳机按钮，人物戴上耳机装饰，再点击一下卸下，这里仅展示几个效果，可以根据自己喜好设置想要的装饰
    if(!_isEarphoneEnable)
    {
        _isEarphoneEnable = YES;
        [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Headset_0001_0001"]];
        // 按钮按下，按钮变为灰色
        sender.selected=YES;
    }
    else{
        _isEarphoneEnable = NO;
        [_character removePackage:ZegoAvatarMakeupTypeHeadset];
        // 再按下时，按钮恢复默认蓝色
        sender.selected=NO;
    }
    
}
//眼镜点击事件
-(void)Addglasses:(UIButton *)sender {
    
    if(!_isGlassEnable)
    {
        _isGlassEnable = YES;
        sender.selected=YES;
        [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Glass_0001_0001"]];
       
    }
    else{
        _isGlassEnable = NO;
        sender.selected=NO;
        [_character removePackage:ZegoAvatarMakeupTypeGlasses];
    }

}
//头发点击事件
-(void)Addhair:(UIButton *)sender {
    if(!_isHairEnable)
    {
        _isHairEnable = YES;
        sender.selected=YES;
        [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Hair_0002_0001"]];
    }
    else{
        _isHairEnable = NO;
        sender.selected=NO;
        [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_Hair_0006_0001"]];
    }

}

//刺青点击事件
-(void)Addfacepaint:(UIButton *)sender {

    if(!_isFacepaintEnable)
    {
        _isFacepaintEnable = YES;
        sender.selected=YES;
        [_character setPackage:[self getPackagePath:@"ZEGOQ_Share_W_Sticker_0005_0001_Basecolor"]];
    }
    else{
        _isFacepaintEnable = NO;
        sender.selected=NO;
        [_character removePackage:ZegoAvatarMakeupTypeFacePaint];
    }

}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/Packages/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}

@end
