//
//  EffectViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2024/1/22.
//

#import "EffectViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>

@interface EffectViewController()

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character1;
@property (nonatomic, strong) id<IZegoCharacter> character2;
@property (nonatomic, strong) UIButton *playAnimBtn;
@property (nonatomic, strong) UIButton *playMultiAnimBtn;
@end

@implementation EffectViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];
    
    //切换按钮
    _playAnimBtn = [self addButtonWithTitle:@"播放特效1" action:@selector(playAnim)];
    [_playAnimBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view.mas_centerX).offset(-10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];

    //切换按钮
    _playMultiAnimBtn = [self addButtonWithTitle:@"播放特效2" action:@selector(playMultiAnim)];
    [_playMultiAnimBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view.mas_centerX).offset(10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}


-(void)playAnim {
    [_avatarView playEffect:[self getPackagePath:@"ZEGO_Scen_MagicCircle_001"] loop:NO effectCallback:^(int errorCode, NSString *effectID) {
        
    }];
}

- (void)playMultiAnim {
    [_avatarView playEffect:[self getPackagePath:@"ZEGO_Scene_FlowerFalling_001"] loop:NO effectCallback:^(int errorCode, NSString *effectID) {
        
    }];
}


#pragma mark - Avatar

// 初始化 avatar
- (void) initAvatar{
    NSLog(@"AnimationViewController init avatar!!");
    
    // 初始化 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 630)];
    [self.view addSubview:_avatarView];
    _avatarView.backgroundColor = [UIColor brownColor];
    
    // 创建两个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character1 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    [_avatarView setCharacter:_character1];
}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/Packages/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}
@end
