 //
//  ViewController.m
//

#import "FaceTrackingViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>

@interface FaceTrackingViewController ()<ZegoAvatarServiceDelegate> {
}

@property (nonatomic, strong) ZegoAvatarService *service;
@property (nonatomic, strong) id<IZegoCharacter> character;
@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) UIButton *faceDetectBtn;
@property (nonatomic, strong) UIButton *voiceDetectBtn;

@end

@implementation FaceTrackingViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 必须等 Service InitSucceed 才能调用 avatar 的接口
    _service = [ZegoAvatarService sharedInstance];
   
    // 初始化按钮
    _faceDetectBtn = [self addButtonWithTitle: @"启动摄像头驱动" action:@selector(actionFaceDetectBtnClick:)];
    [_faceDetectBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view).offset(20);
        make.top.mas_equalTo(340);
    }];
    
    _voiceDetectBtn = [self addButtonWithTitle: @"启动麦克风驱动" action:@selector(actionVoiceDetectBtnClick:)];
    [_voiceDetectBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view).offset(-20);
        make.top.mas_equalTo(340);
    }];
    
    // 初始化Avatar
    [self initAvatar];
    
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [self stopDetect];
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}

#pragma mark - Avatar

- (void) initAvatar{
    
    // 创建 avatarView
    _avatarView = [_service createAvatarView:CGRectMake((self.view.bounds.size.width - 200) / 2/*居中*/, getRectNavAndStatusHight + 40, 200, 200)];
    [self.view addSubview:_avatarView];
    
    // 创建一个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    
    // 把角色渲染到 view 上，不然不可见
    [_avatarView setCharacter:_character];
    
    // 开启表情需要关闭动画
    [_character enableAnimation:NO];
        
    // 展示表情随动效果，视角切换到头部更清晰
    [_avatarView setCamera:@"head"];
    
}

- (void ) startDetect: (ZegoExpressionDetectMode) mode{
    __weak typeof(self) weakSelf = self;
    
    [[_service getInteractEngine] startDetectExpression: mode callback:^(ZegoExpression *expression) {
        [self.character setExpression:expression];
    }];
}

-(void) stopDetect{
    [[_service getInteractEngine] stopDetectExpression];
}

#pragma mark - listener

-(void)actionFaceDetectBtnClick:(UIButton *)sender {
    [self stopDetect];
    [self startDetect: ZegoExpressionDetectModeCamera];
    [_faceDetectBtn setEnabled: NO];
    [_voiceDetectBtn setEnabled: YES];
}

-(void)actionVoiceDetectBtnClick:(UIButton *)sender {
    [self stopDetect];
    [self startDetect: ZegoExpressionDetectModeAudio];
    [_faceDetectBtn setEnabled: YES];
    [_voiceDetectBtn setEnabled: NO];
}


@end
