//
//  RecordGIFViewController.m
//  ZegoAvatarExample
//
//  Created by zego on 2023/11/15.
//

#import "RecordGIFViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>
#import <FLAnimatedImage/FLAnimatedImage.h>


@interface RecordGIFViewController()

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character1;
@property (nonatomic, strong) UIButton *recordGIF;
@property (nonatomic, strong) UIButton *stopRecordGIF;
@property (nonatomic, strong) UIButton *changeAnimation;
@property (nonatomic, strong) FLAnimatedImageView *gifView;
@property (nonatomic, strong) NSArray *stringArray;
@end

@implementation RecordGIFViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    _stringArray = @[@"ZEGOQ_Man_Action_Greeting_0001", @"ZEGOQ_Man_Action_Blowkiss_0001", @"ZEGOQ_Man_Action_Breathe_0001", @"ZEGOQ_Man_Action_To_Hug_0001",  @"ZEGOQ_Man_Action_Get_Changed_0001"];
    
    // 初始化 Avatar
    [self initAvatar];
    
    //切换按钮
    _recordGIF = [self addButtonWithTitle:@"开始录制GIF" action:@selector(recordGIFClick)];
    [_recordGIF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view.mas_centerX).offset(-10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
    _recordGIF.enabled = YES;
    
    _stopRecordGIF = [self addButtonWithTitle:@"停止录制GIF" action:@selector(stopRecordGIFClick)];
    [_stopRecordGIF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view.mas_centerX).offset(10);
        make.bottom.equalTo(self.view.mas_bottom).offset(-60);
    }];
    _stopRecordGIF.enabled = NO;
    
    //切换按钮
    _changeAnimation = [self addButtonWithTitle:@"切换动作" action:@selector(changeAnimationClick)];
    [_changeAnimation mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.centerX.equalTo(self.view.mas_centerX).offset(0);
        make.bottom.equalTo(self.view.mas_bottom).offset(-120);
    }];

    self.gifView = [[FLAnimatedImageView alloc] init];
    [self.view addSubview:self.gifView];
    [self.gifView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.view.mas_top).offset(90);
        make.right.equalTo(self.view.mas_right).offset(-30);
        make.width.mas_equalTo(110);
        make.height.mas_equalTo(110);
    }];
    self.gifView.backgroundColor = [UIColor whiteColor];
    _gifView.hidden = YES;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}


-(void)changeAnimationClick {
    // 生成一个随机索引
    NSUInteger randomIndex = arc4random_uniform((uint32_t)[self.stringArray count]);
    // 使用索引获取随机字符串
    NSString *randomString = self.stringArray[randomIndex];
    [_character1 playAnimation:[self getPackagePath:randomString]];
}

-(void)recordGIFClick {
    [_avatarView startRecordFramePack];
    _gifView.hidden = YES;
    _recordGIF.enabled = NO;
    _stopRecordGIF.enabled = YES;
}

-(void)stopRecordGIFClick {
    _stopRecordGIF.enabled = NO;
    [_avatarView stopRecordFramePack:^(int errorCode, NSString *path) {
        NSLog(@"path: %@", path);

        self->_gifView.hidden = NO;
        // 从文件路径创建 FLAnimatedImage 实例
        NSURL *gifFileURL = [NSURL fileURLWithPath:path];
        FLAnimatedImage *animatedImage = [FLAnimatedImage animatedImageWithGIFData:[NSData dataWithContentsOfURL:gifFileURL]];
        self->_gifView.animatedImage = animatedImage;
        self->_gifView.contentMode = UIViewContentModeScaleAspectFit;
        self->_gifView.clipsToBounds = YES;
        
        self->_recordGIF.enabled = YES;
        self->_stopRecordGIF.enabled = NO;
    }];
}


#pragma mark - Avatar

// 初始化 avatar
- (void) initAvatar{
    NSLog(@"AnimationViewController init avatar!!");
    
    // 初始化 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 630)];
    [self.view addSubview:_avatarView];
    [_avatarView setBackgroundColor:[UIColor redColor]];
    // 创建两个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character1 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    [_avatarView setCharacter:_character1 callback:^{
       // 上屏回调
    }];
}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/Packages/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}
@end
