//
//  SceneViewController.m
//  ZegoAvatarExample
//
//  Created by xyl on 2023/11/8.
//

#import "SceneViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>

@interface SceneViewController()

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character1;
@property (nonatomic, strong) UIButton *sceneBtn;
@property (nonatomic, strong) UIButton *removeSceneBtn;
@property (nonatomic, strong) UIButton *scaleSceneBtn;
@end

@implementation SceneViewController
- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];
    
    _sceneBtn = [self addButtonWithTitle:@"场景" action:@selector(setScene)];
    [_sceneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view).offset(20);
        make.top.mas_equalTo(640);
    }];
    _sceneBtn.enabled = YES;
    
    
    _removeSceneBtn = [self addButtonWithTitle:@"移除场景" action:@selector(removeScene)];
    [_removeSceneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view).offset(-20);
        make.top.mas_equalTo(640);
    }];
    _removeSceneBtn.enabled = YES;
    

    _scaleSceneBtn = [self addButtonWithTitle:@"缩放场景" action:@selector(scaleScene)];
    [_scaleSceneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
    make.width.mas_equalTo(120);
    make.height.mas_equalTo(44);
    make.centerX.equalTo(self.view);
    make.top.mas_equalTo(700);
    }];
    _scaleSceneBtn.enabled = YES;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}

-(void)setScene {
    NSString* scenePath = [self getPackagePath:@"ZEGOQ_CustomScene_0001"];
    [_avatarView setScene:scenePath callback:^(int errorCode) {
        NSLog(@"SceneViewController setScene errorCode %d", errorCode);
    }];
}

-(void)removeScene {
    [_avatarView removeScene];
}

-(void)scaleScene {
    [_avatarView setSceneScale:0.5 y:0.5 z:0.5];
}

#pragma mark - Avatar

// 初始化 avatar
- (void) initAvatar{
    NSLog(@"SceneViewController init avatar!!");
    
    // 初始化 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 630)];
    [self.view addSubview:_avatarView];
    _avatarView.backgroundColor = [UIColor brownColor];
    
    // 创建两个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character1 = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    [_avatarView setCharacter:_character1];
}

-(NSString*) getPackagePath: (NSString* )packageID {
    NSString* bathPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/"];
    NSString* packagePath = [bathPath stringByAppendingString:packageID];
    return packagePath;
}
@end
