//
//  TextDriveViewController.m
//  ZegoAvatarExample
//
//  Created by lihua.wei on 2023/2/19.
//

#import "TextDriveViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>
#import <AVFAudio/AVAudioSession.h>

#import "ZegoTextLicenseCustomer.h"
#import "ZegoTextAPI.h"
#import "ZegoAvatarConfig.h"

#define ScreenWidth [UIScreen mainScreen].bounds.size.width
#define ScreenHeight [UIScreen mainScreen].bounds.size.height

@interface TextDriveViewController () <ZegoAvatarServiceDelegate, UITextViewDelegate>

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character;

@property (nonatomic, strong) UIView *inputBackgroundView;
@property (nonatomic, strong) UITextView *textView;
@property (nonatomic, assign) CGFloat keyboardHeight;

@property (nonatomic, strong) ZegoTextAPI *textApi;

@end

@implementation TextDriveViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setBackground];

    [ZegoTextLicenseCustomer setZegoTextAppID:AVATAR_APPID];
    [ZegoTextLicenseCustomer setZegoTextAppSign:AVATAR_APP_SIGN];

    CGFloat width = [UIScreen mainScreen].bounds.size.width;
    //736
    CGFloat height = [UIScreen mainScreen].bounds.size.height - 40 - 54;

    self.inputBackgroundView.frame=CGRectMake(0, height, width, 54);
    [self.view addSubview:self.inputBackgroundView];

    self.textView.frame=CGRectMake(15, 12, width-30, 30);
    [self.inputBackgroundView addSubview:self.textView];

    //键盘的frame即将发生变化时立刻发出该通知
    [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(keyboardChanged:) name:UIKeyboardWillChangeFrameNotification object:nil];

    AVAudioSession *avSession = [AVAudioSession sharedInstance];
    [avSession setCategory:AVAudioSessionCategoryPlayback withOptions:AVAudioSessionCategoryOptionDefaultToSpeaker error:nil];
    [avSession setActive:YES error:nil];
    
    // 初始化 Avatar
    [self initAvatar];
    
    [self.view bringSubviewToFront:self.inputBackgroundView];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}

- (void) initAvatar{
    // 创建 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 30, self.view.bounds.size.width, self.view.bounds.size.height - 100)];
    [self.view addSubview:_avatarView];
    
    // 创建一个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    
    // 把角色渲染到 view 上，不然不可见
    [_avatarView setCharacter:_character];
    
    [_character enableAnimation:NO];
}

- (void)keyboardChanged:(NSNotification *)notification{
    
    CGRect frame=[notification.userInfo[UIKeyboardFrameEndUserInfoKey]CGRectValue];
    
    CGRect currentFrame=self.inputBackgroundView.frame;
    
    [UIView animateWithDuration:0.25 animations:^{
        //输入框最终的位置
        CGRect resultFrame;
        
        if (frame.origin.y==ScreenHeight) {
            resultFrame=CGRectMake(currentFrame.origin.x, ScreenHeight - 54 - 40, currentFrame.size.width, 54);
            self.keyboardHeight=-40;
        }else{
            resultFrame=CGRectMake(currentFrame.origin.x,ScreenHeight - 54 - frame.size.height , currentFrame.size.width, 54);
            self.keyboardHeight=frame.size.height;
        }
        
        self.inputBackgroundView.frame=resultFrame;
        self.textView.frame=CGRectMake(15, 12, ScreenWidth-30, 30);
    }];
    
}

- (void)textViewDidChange:(UITextView *)textView{
    
    NSString *str=textView.text;
    
    if (str.length > 300) {
        str = [str substringWithRange:NSMakeRange(0, 300)];
        textView.text = str;
    }
    
    CGSize maxSize=CGSizeMake(textView.bounds.size.width, MAXFLOAT);
    //测量string的大小
    CGRect frame=[str boundingRectWithSize:maxSize options:NSStringDrawingUsesLineFragmentOrigin|NSStringDrawingUsesFontLeading attributes:@{NSFontAttributeName:textView.font} context:nil];
    //设置self.textView的高度，默认是30
    CGFloat tarHeight=30;
    //如果文本框内容的高度+10大于30也就是初始的self.textview的高度的话，设置tarheight的大小为文本的内容+10，其中10是文本框和背景view的上下间距；
    if (frame.size.height+10>30) {
        tarHeight=frame.size.height+10;
    }
    //如果self.textView的高度大于200时，设置为200，即最高位200
    if (tarHeight>92) {
        tarHeight=92;
    }
    
    CGFloat width=ScreenWidth;
    //设置输入框背景的frame
    self.inputBackgroundView.frame=CGRectMake(0, (ScreenHeight-self.keyboardHeight)-(tarHeight+22), width, tarHeight+22);
    //设置输入框的frame
    self.textView.frame=CGRectMake(15,(self.inputBackgroundView.bounds.size.height-tarHeight)/2 , width-30, tarHeight);
}

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text
{
    if ([text isEqualToString:@"\n"]) {
        [textView resignFirstResponder];
        
        [self.textApi playTextExpression:textView.text];
        
        textView.text = nil;
        return NO;
    }
    return YES;
}

- (void)setBackground {
    UIImage *image = [UIImage imageNamed:@"image_bg"];
    self.view.layer.contents = (id) image.CGImage;
    self.view.layer.backgroundColor = [UIColor clearColor].CGColor;
}

- (UIView *)inputBackgroundView {
    if (_inputBackgroundView==nil) {
        _inputBackgroundView=[UIView new];
        _inputBackgroundView.backgroundColor=[UIColor whiteColor];
    }
    return _inputBackgroundView;
}

- (UITextView *)textView {
    if (_textView==nil) {
        _textView=[[UITextView alloc]init];
        _textView.backgroundColor = [UIColor whiteColor];
        _textView.textColor = [UIColor blackColor];
        _textView.returnKeyType = UIReturnKeyDone;
        
        UILabel *placeHolderLabel = [[UILabel alloc] init];
        placeHolderLabel.text = @"请输入文字驱动虚拟人播报…";
        placeHolderLabel.numberOfLines = 0;
        placeHolderLabel.textColor = [UIColor lightGrayColor];
        [placeHolderLabel sizeToFit];
        [_textView addSubview:placeHolderLabel];
        
        // same font
        _textView.font = [UIFont systemFontOfSize:14.f];
        placeHolderLabel.font = [UIFont systemFontOfSize:14.f];
        [_textView setValue:placeHolderLabel forKey:@"_placeholderLabel"];
        _textView.delegate=self;
    }
    return _textView;
}

- (ZegoTextAPI *)textApi {
    if (!_textApi) {
        _textApi = [[ZegoTextAPI alloc] initWithCharacter:self.character];
    }
    return _textApi;
}

@end
