 //
//  ViewController.m
//

#import "VideoRecordingViewController.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>
#import <Photos/Photos.h>
#import "UIView+Toast.h"

@interface VideoRecordingViewController ()<ZegoAvatarServiceDelegate> {
}

@property (nonatomic, strong) UIView <IZegoAvatarView> *avatarView;
@property (nonatomic, strong) id<IZegoCharacter> character;
@property (nonatomic, strong) UIButton *startBtn;
@property (nonatomic, strong) UIButton *stopBtn;
@property (nonatomic, strong) UIButton *screenshotBtn;
@property (nonatomic, strong) NSString *videoPath;

@end

@implementation VideoRecordingViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化 Avatar
    [self initAvatar];
    
    // 初始化按钮
    _startBtn = [self addButtonWithTitle: @"开始录制" action:@selector(actionStartBtnClick:)];
    [_startBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.left.equalTo(self.view).offset(20);
        make.top.mas_equalTo(540);
    }];
    
    _stopBtn = [self addButtonWithTitle: @"停止录制" action:@selector(actionStopBtnClick:)];
    [_stopBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.right.equalTo(self.view).offset(-20);
        make.top.mas_equalTo(540);
    }];
    [_stopBtn setEnabled:false];
    
    
    _screenshotBtn = [self addButtonWithTitle: @"截一张图" action:@selector(actionScreenshotBtnClick:)];
    [_screenshotBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(44);
        make.centerX.equalTo(self.view);
        make.top.mas_equalTo(600);
    }];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    // 回收 Avatar
    [self stopDetect];
    [_avatarView removeAllCharacter];
    _avatarView = nil;
}

#pragma mark - Avatar

- (void) initAvatar{
    // 创建 avatarView
    _avatarView = [[ZegoAvatarService sharedInstance] createAvatarView:CGRectMake(0, 0, self.view.bounds.size.width, 400)];
    [self.view addSubview:_avatarView];
    // 设置背景颜色
    _avatarView.backgroundColor = [UIColor cyanColor];

    // 创建一个 avatar 角色
    NSString *assetBundlesPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/qhuman.bundle/AssetBundles/bundle/qbody/Boy.prefab"];
    _character = [[ZegoAvatarService sharedInstance] createCharacter:assetBundlesPath];
    
    // 把角色渲染到 view 上，不然不可见
    [_avatarView setCharacter:_character];
    
    // 把表情也录制一下，所以这里停止动画
    [_character enableAnimation:NO];
    
    // 启动表情随动
    [self startDetect];
}

- (void ) startDetect{
    __weak typeof(self) weakSelf = self;
    
    [[[ZegoAvatarService sharedInstance] getInteractEngine] startDetectExpression: ZegoExpressionDetectModeCamera callback:^(ZegoExpression *expression) {
        __strong typeof(self) he = weakSelf;
        [he->_character setExpression: expression];
    }];
}

-(void) stopDetect{
    [[[ZegoAvatarService sharedInstance] getInteractEngine] stopDetectExpression];
}

#pragma mark - listener

-(void)actionStartBtnClick:(UIButton *)sender {
    NSString *docDir = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject;
    _videoPath = [NSString stringWithFormat:@"%@/avatar.mov", docDir];
    // 拼装路径
    ZegoRecordConfig *config = [ZegoRecordConfig new];
    config.videoPath = _videoPath;
    // 录制 mp4 (h264编码)
//    config.recordMode = ZegoAvatarRecordModeMP4;
    __weak typeof(self) weakSelf = self;
    // 启动录制, 由于录制是在子线程进行的, 当启动失败返回false, 录制过程中有错误(找不到编码器,或帧编码失败等错误), 会通过 callback 回调出来
    BOOL success = [_avatarView startRecord:config onStartRecordCallback:^(NSInteger errorCode, NSString *info) {
        if(errorCode != ZegoAvatarErrorCodeSuccess){
            __strong typeof(self) he = weakSelf;
            [he.view makeToast: [NSString stringWithFormat: @"录制出错, code: %ld, msg: %@", (long)errorCode, info]];
        }
    }];
    if (success) {
        [self.view makeToast:@"录制启动成功"];
        [_startBtn setEnabled: NO];
        [_stopBtn setEnabled: YES];
    }
    
}

-(void)actionScreenshotBtnClick:(UIButton *)sender {
    UIImage *img = [self.avatarView captureScreenshot];
    // 存到相册，请确保已经有权限了，这部分可自由实现
    UIImageWriteToSavedPhotosAlbum(img, nil, nil, nil);
}

-(void)actionStopBtnClick:(UIButton *)sender {
    __weak typeof(self) weakSelf = self;
    [_avatarView stopRecord:^{
        __strong typeof(self) he = weakSelf;
        if (UIVideoAtPathIsCompatibleWithSavedPhotosAlbum(he.videoPath)) {
            //保存相册核心代码
            // ios 12以前的系统, 要加上这个权限: Privacy - Photo Library Additions Usage Description
            UISaveVideoAtPathToSavedPhotosAlbum(he.videoPath, he, @selector(video:didFinishSavingWithError:contextInfo:), nil);
        }
    }];
    [_startBtn setEnabled: YES];
    [_stopBtn setEnabled: NO];
}

//保存视频完成之后的回调
- (void)video:(NSString *)videoPath didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo {
    NSString *tip = error ? @"Avatar形象视频保存失败" : @"Avatar形象视频已保存到相册";
    dispatch_async(dispatch_get_main_queue(), ^{
        CSToastStyle *style = [[CSToastStyle alloc] initWithDefaultStyle];
        style.messageFont = [UIFont systemFontOfSize:14];
        style.messageColor = [UIColor whiteColor];
        style.messageAlignment = NSTextAlignmentCenter;
        style.backgroundColor = UIColor.blueColor;
        style.cornerRadius = 18.5;
        
        [self.view makeToast:tip
                    duration:2.0
                    position:CSToastPositionTop
                       style:style];
    });
}


@end
