//
//  ViewController.m
//  ZegoAvatarExample
//
//  Created by Azrael Long on 2/6/2022.
//

#import "ViewController.h"
#import "ZegoHudManager.h"
#import <ZegoAvatar/ZegoAvatarService.h>
#import <Masonry/Masonry.h>
#import "FaceTrackingViewController.h"
#import "VideoRecordingViewController.h"
#import "TextDriveViewController.h"
#import "ZegoAvatarConfig.h"

@interface ViewController () <ZegoAvatarServiceDelegate, UITableViewDataSource,UITableViewDelegate>

@property (nonatomic, strong) UILabel *sdkVersionLabel;
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic) NSMutableArray *tableData;
@property (strong, nonatomic) NSMutableDictionary *modules;
@end

@implementation ViewController

// avatarService初始化状态回调
- (void)onStateChange:(ZegoAvatarServiceState)state {
    // SDK初始化成功
    if (state == ZegoAvatarServiceStateInitSucceed || state == ZegoAvatarServiceStateInitFailed) {
        [ZegoHudManager hideNetworkLoading];
    }
}

- (void) onError:(ZegoAvatarErrorCode)code desc:(NSString *)desc{
    NSLog(@"Avatar Error: %ld, desc: %@", (long)code, desc);
}


- (void)viewDidLoad {
    [super viewDidLoad];
    
    _modules = [NSMutableDictionary dictionary];
    _tableData = [NSMutableArray array];
    
    [ZegoHudManager showNetworkLoading];
    
    [self initAvatarService];
    
    // 获取SDK版本号
    UILabel *sdkVersionLabel = [[UILabel alloc] init];
    sdkVersionLabel.font = [UIFont systemFontOfSize:12];
    sdkVersionLabel.textColor = UIColor.blackColor;
    sdkVersionLabel.textAlignment = NSTextAlignmentCenter;
    [self.view addSubview:sdkVersionLabel];
    [sdkVersionLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.bottom.equalTo(self.view).offset(-40);
        make.width.mas_equalTo(200);
        make.height.mas_equalTo(sdkVersionLabel.font.lineHeight);
    }];
    self.sdkVersionLabel = sdkVersionLabel;
    sdkVersionLabel.text = [ZegoAvatarService getVersion];

    // 添加列表项
    [self addView:@"FaceTrackingViewController" withName:@"表情随动"];
    [self addView:@"ManualFaceViewController" withName:@"捏脸塑型"];
    [self addView:@"DressupViewController" withName:@"妆容换装"];
    [self addView:@"AnimationViewController" withName:@"播放动画"];
    [self addView:@"MultiAvatarViewController" withName:@"多个 AvatarView"];
    [self addView:@"VideoRecordingViewController" withName:@"Avatar 录制"];
    [self addView:@"PushPullFlowViewController" withName:@"Avatar 推流"];
    [self addView:@"TextDriveViewController" withName:@"文本驱动"];
    [self addView:@"SceneViewController" withName:@"3D场景"];
    [self addView:@"CameraMotionViewController" withName:@"播放运镜"];
    [self addView:@"RecordGIFViewController" withName:@"录制表情包"];
    [self addView:@"EffectViewController" withName:@"3D特效"];
    [self addView:@"AttachmentViewController" withName:@"人物挂饰"];

    // 设置 table view
    self.tableView.dataSource = self;
    self.tableView.delegate = self;
    self.tableView.backgroundColor = [UIColor whiteColor];
}

- (void)initAvatarService {
    ZegoServiceConfigV2* config = [[ZegoServiceConfigV2 alloc]init];
    config.appID = AVATAR_APPID;
    config.appSign = AVATAR_APP_SIGN;
    config.AIPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/AIModel.bundle"];;
    config.unityBundleID = @"com.zego.zegoavatar";
    // 初始化ZegoAvatarService
    [[ZegoAvatarService sharedInstance] addServiceObserver:self];
    [[ZegoAvatarService sharedInstance] initWithConfigV2:config];
}

- (void) addView: (NSString*) vc withName: (NSString*) name{
    [_modules setObject:vc forKey:name];
    [_tableData addObject: name];
}

- (void)viewWillDisappear:(BOOL)animated{
    // 退出页面要remove
    [[ZegoAvatarService sharedInstance] removeServiceObserver:self];
}

// 初始化 avatar Service
- (void) initAvatarService: (NSString*) license{
    // 传入鉴权文件，创建 ZegoAvatarService 对象
    ZegoServiceConfig *config = [[ZegoServiceConfig alloc] init];
    // License
    config.license = license;
    // 指定ai模型的路径
    config.AIPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/assets/AIModel.bundle"];
    // 初始化ZegoAvatarService
    [[ZegoAvatarService sharedInstance] addServiceObserver:self];
    [[ZegoAvatarService sharedInstance] initWithConfig:config];
    
}

// 每行显示的内容是什么
- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    // 从重用池中查找对应的cell
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier: @"ABC"];
    if(cell == nil){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ABC"];
    }
    cell.textLabel.text= [NSString stringWithFormat:@"%ld. %@", (long)indexPath.row, self.tableData[indexPath.row]];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    cell.contentView.backgroundColor = [UIColor whiteColor];
    cell.backgroundColor = [UIColor whiteColor];
    cell.textLabel.textColor = [UIColor blackColor];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    return cell;
}

// 一组中有多少行
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.tableData.count;
}

// 有多少组,默认为一组
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

// 代理table的点击事件
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    NSInteger index = indexPath.row;
    NSString *title= _tableData[index];
    NSString *vcName = _modules[title];
    NSLog(@"Click Title: %@, VC: %@", title, vcName);
    Class cls = NSClassFromString(vcName);
    BaseViewController *vc = [[cls alloc] initWithTitle: title];
    if(vc != nil){
        [self.navigationController pushViewController:vc animated:YES];
    }
}




@end
